package flowserver

import (
	"flow/registry"
	"log"
	"net/http"
	"net/http/httputil"
	"net/url"
	"os"

	"github.com/gohxs/prettylog"
	"github.com/gohxs/webu"
)

//go:generate go get github.com/gohxs/genversion
//go:generate genversion -package flowserver -out version.go
//

// FlowServer structure
type FlowServer struct {
	//mux            *http.ServeMux
	sessionHandler http.Handler
	staticHandler  http.Handler
}

// New creates a New flow server
func New(r *registry.R, store string) *FlowServer {
	if r == nil {
		r = registry.Global.Clone()
	}
	var sessionHandler http.Handler
	var staticHandler http.Handler

	sessionHandler = NewFlowSessionManager(r, store)

	if os.Getenv("DEBUG") == "1" {
		log.Println("DEBUG MODE: reverse proxy localhost:8081")
		proxyURL, err := url.Parse("http://localhost:8081")
		if err != nil {
			return nil
		}

		rp := httputil.NewSingleHostReverseProxy(proxyURL)
		rp.ErrorLog = prettylog.New("rproxy")
		staticHandler = rp
	} else {
		staticHandler = webu.StaticHandler("web", "index.html")
	}

	/*mux := http.NewServeMux()
	mux.Handle("/conn", sessionHandler)
	mux.Handle("/", staticHandler)*/

	return &FlowServer{
		sessionHandler: sessionHandler,
		staticHandler:  staticHandler,
	}
}

func (f *FlowServer) ServeHTTP(w http.ResponseWriter, r *http.Request) {
	log.Println("Requesting for:", r.URL)

	// Manual routing
	switch r.URL.Path {
	case "/conn":
		f.sessionHandler.ServeHTTP(w, r)
	default:
		f.staticHandler.ServeHTTP(w, r)
	}
	//handler, _ := f.mux.Handler(r)
	//handler.ServeHTTP(w, r)
	//f.mux.ServeHTTP(w, r)
	// Manual routing here
	// Grab last part of path
	/*prefixToRemove := ""

	urlParts := strings.Split(r.URL.Path, "/")
	if len(urlParts) > 1 {
		//prefixToRemove = strings.Join(urlParts[:len(urlParts)-1], "/")
		prefixToRemove = strings.TrimRight("/"+urlParts[1], "/")
		log.Println("Will remove", prefixToRemove)
	}
	//	}

	http.StripPrefix(prefixToRemove, f.mux).ServeHTTP(w, r)*/
}

// ListenAndServe starts the httpserver
// It will listen on default port 2015 and increase if port is in use
func (f *FlowServer) ListenAndServe(addr string) error {
	return http.ListenAndServe(addr, f)
}
